/**
 * CopyRight (C) 2005 Bam Rescue Simulation Team.All rights resserved.
 *
 * Hamid Mohammadi <hmohammadi110@gmail.com>
 * Hamed Shahbazi  <demahopilot@yahoo.com>
 * Behrooz Shahgholi <shahgholi@eng.ui.ac.ir>
 *
 */

package bam;

import java.net.*;
import java.util.*;
import yab.agent.Agent;
import yab.io.ProtocolConstants;

public final class Main implements ProtocolConstants {
    private static InetAddress m_address;
    private static int m_port;

    private static final int
            AT = 0, AC = 1, NUM_TYPE = 2;
    private static final int[] m_num = new int[NUM_TYPE];
    private static boolean noMoreAgent = false;

    public static void main(String[] args) {
        parseArgs(args);
        List agents = new ArrayList();
        connectAgents(AT, agents);
        connectAgents(AC, agents);
        yab.io.RCRSSProtocolSocket.clearUpAfterInitialization();
        for (Iterator it = agents.iterator(); it.hasNext(); ) {
            ((Agent) it.next()).start();
        }
    }

    private static void connectAgents(int type, List agents) {
        noMoreAgent = false;
        for (int i = 0; i < m_num[type] && !noMoreAgent; i++) {
            try {
                agents.add(connectAgent(type));
            } catch (Error e) {
                String reason = e.getMessage();
                if (reason.equals(REASON_OF_KA_CONNECT_ERROR)
                    && m_num[type] == Integer.MAX_VALUE) {
                    break;
                } else if (reason.equals(REASON_OF_KA_CONNECT_ERROR)) {
                    System.out.println("End of Connecting agents...");
                    noMoreAgent = true;
                    break;
                }

                System.err.println(
                        "End of Connecting...\n Mesage from Kernel: " + reason);
                noMoreAgent = true;
            }
        }
    }

    private static Agent connectAgent(int type) {
        switch (type) {
        default:
            throw new Error();
        case AT:
            return new AmbulanceTeamAgent(m_address, m_port);
        case AC:
            return new AmbulanceCenterAgent(m_address, m_port);
        }
    }

    private static void parseArgs(String[] args) {
        try {
            m_address = InetAddress.getByName("localhost");
            m_port = KERNEL_LISTENING_PORT;
            if (args.length == 0) {
                for (int i = 0; i < NUM_TYPE; i++) {
                    m_num[i] = Integer.MAX_VALUE;
                }
                return;
            }
            if (args.length < NUM_TYPE) {
                printUsage();
            }
            for (int i = 0; i < NUM_TYPE; i++) {
                m_num[i] = (args[i].equals("-"))
                           ? Integer.MAX_VALUE
                           : Integer.parseInt(args[i]);
            }

            if (args.length < NUM_TYPE + 1) {
                return;
            }
            m_address = InetAddress.getByName(args[NUM_TYPE]);

            if (args.length < NUM_TYPE + 2) {
                return;
            }
            m_port = Integer.parseInt(args[NUM_TYPE + 1]);

            if (args.length > NUM_TYPE + 2) {
                printUsage();
            }
        } catch (Exception e) {
            printUsage();
        }
    }

    private static void printUsage() {
        String usage
                = "usage:\n"
                  + "  java samaple.Main "
                  + "[<at> <ac> [<host> [<port>]]]\n"
                  + "    <at>   the number of ambulance team agents or '-'\n"
                  + "    <ac>   the number of ambulance center agents or '-'\n"
                  + "    <host> kernel host (defalut : localhost)\n"
                  + "    <port> kernel port (default : " +
                  KERNEL_LISTENING_PORT + ")\n"
                  +
                "  If the numbers com.waldura.twof agents are not given, this program tries to"
                  + " connect all\n"
                  + "  type agents as much as the kernel accepts.\n";
        System.err.print(usage);
        System.exit(1);
    }
}
