/**
 * CopyRight (C) 2005 Bam Rescue Simulation Team.All rights resserved.
 *
 * Hamid Mohammadi <hmohammadi110@gmail.com>
 * Hamed Shahbazi  <demahopilot@yahoo.com>
 * Behrooz Shahgholi <shahgholi@eng.ui.ac.ir>
 *
 */


package bam;

import java.net.*;
import java.util.*;
import yab.agent.*;
import yab.agent.object.*;

public class FireBrigadeAgent extends AbstractFireBrigadeAgent {

    public FireBrigadeAgent(InetAddress address, int port) {

        super(address, port);

        searchBuildings = new ArrayList(world.buildings);

    }

    private ArrayList reportBlockadeOthers = new ArrayList();
    private ArrayList brdReports = new ArrayList();
    private ArrayList nearFires1 = new ArrayList();
    private ArrayList searchBuildings = new ArrayList();
    private ArrayList randompoint = new ArrayList();

    private int randtime = 0;
    protected static final int BURNING_TIME_OF_AN_EARLY_FIRE = 9;


    protected static final Property
            BURIEDNESS_PRP = Property.get("Humanoid", "buriedness"),
                             DAMAGE_PRP = Property.get("Humanoid", "damage"),
                                          HP_PRP = Property.get("Humanoid",
            "hp"),
            FIERYNESS_PRP = Property.get("Building", "fieryness"),
                            BURNING_TIME_PRP = Property.get("Building",
            "burningTime"),
                                               POSITION_PRP = Property.get(
            "MovingObject", "position"),
            POSITION_BUILDING_PRP = Property.get("MotionlessObject",
                                                 "motionlessPosition"),
                                    KIND_PRP = Property.get("Building",
            "buildingCode"),
                                               passLine = Property.get("Road",
            "passableLines"),
            ARIA_PRP = Property.get("Building", "buildingAreaTotal");

    protected static final Condition
            IS_BURNING_CND = FIERYNESS_PRP.gte(1).and(FIERYNESS_PRP.lte(3)),
                             IS_BURIEDNESS_CND = BURIEDNESS_PRP.gt(0),
                                                 IS_NOT_BURNING_CND =
            FIERYNESS_PRP.lte(0),
            FIRST_RATE_BURNING_CND = FIERYNESS_PRP.gte(2).and(FIERYNESS_PRP.lte(
            3)),
                                     KIND_WOODY = KIND_PRP.lte(1).or(ARIA_PRP.
            lt(500)),
                                                  CTRLD_FIRES = FIERYNESS_PRP.
            gte(4),
            passCnd = passLine.lt(1),
                      IS_EARLY_CND = BURNING_TIME_PRP.lte(
                              BURNING_TIME_OF_AN_EARLY_FIRE);

    protected final Condition
            isNearCnd = distancePrp.lte(EXTINGUISHABLE_DISTANCE);


    protected void act() throws ActionCommandException {

        checkBloackade();

        if (self().buriedness() != 0) {
            tell("fbh " + self().hp() + " " + self().damage() + " " +
                 self().buriedness() + " " + self().id + " " +
                 self().motionlessPosition().id);
        } else {
            reportBlockade();
            reportburriedness();
        }

        supplyWater();

        doExtinguish();

        moveRandom();
    }


    protected void doExtinguish() throws ActionCommandException {

        nearFires1 = IS_BURNING_CND.extract(world.buildings);
        ArrayList fires = new ArrayList(nearFires1);
        int ID = 0;
        int maxArea = 0;
        Building temp;
        for (Iterator it = nearFires1.iterator(); it.hasNext(); ) {
            temp = (Building) it.next();
            if (temp.safeNeighbor() != 0) {
                if (temp.safeNeighbor() -
                    self().motionlessPosition().distance(temp) / 18 > maxArea) {
                    ID = temp.id;
                    maxArea = temp.safeNeighbor() -
                              self().motionlessPosition().distance(temp) / 18;
                }
            }
        }
        temp = (Building) world.get(ID);
        if (temp != null) {
            ArrayList test = new ArrayList();
            test.add(temp);
            extinguishOrMove(test);
        } else if (!fires.isEmpty()) {

            extinguishOrMove(fires);
        }

    }


    protected void moveRandom() throws ActionCommandException {
        if (!randompoint.isEmpty()) {
            if (self().motionlessPosition().equals((MotionlessObject)
                    randompoint.get(0)) || (time() - randtime) > 4) {
                randompoint.clear();
            } else {
                move(randompoint);
            }
        }
        if (!searchBuildings.isEmpty()) {
            if (time() <= 10) {
                randompoint.clear();
            }

            Building randmChoice = (Building) searchBuildings.remove(0);
            if (randmChoice.isUnburned()) {
                randompoint.add(randmChoice);
                randtime = time();

                move(randompoint);
            }
        }

    }

    protected void reportBlockade() throws ActionCommandException {
        MotionlessObject pos = self().motionlessPosition();
        ArrayList fir = new ArrayList();
        if (pos instanceof Road && ((Road) pos).passableLinesToHead() == 0 &&
            ((Road) pos).passableLinesToTail() == 0) {

            Road tempRoad = (Road) world.get(pos.id);
            if (tempRoad.directiontotail == 1) {
                tempRoad.incEngagedToTail(2);
            } else {
                tempRoad.incEngagedToHead(2);
            }
            if (!reportBlockadeOthers.contains(tempRoad)) {
                tell("nmv " + pos.id);
            }
            List nearfires2 = IS_BURNING_CND.extract(world.buildings);
            fir.clear();
            for (Iterator it1 = nearfires2.iterator(); it1.hasNext(); ) {
                Building temp = (Building) it1.next();
                if (self().motionlessPosition().distance(temp) <
                    2 * EXTINGUISHABLE_DISTANCE) {
                    fir.add(temp);
                }
            }
            if (fir.isEmpty() && isNear(world.nearPolice)) {
                ArrayList near = new ArrayList();
                near.addAll(world.buildings);
                near.remove(self().motionlessPosition());
                move(near);
            }
        } else if (pos instanceof Road && ((Road) pos).passableLines() == 0) {
            Road tempRoad = (Road) world.get(pos.id);
            if (tempRoad.directiontotail == 1) {
                tempRoad.incEngagedToTail(2);
            } else {
                tempRoad.incEngagedToHead(2);
            }
            if (!reportBlockadeOthers.contains(tempRoad)) {
                tell("clr " + pos.id);
            }
        }
    }

    protected boolean isNear(ArrayList polices) {

        for (Iterator itr12 = polices.iterator(); itr12.hasNext(); ) {
            Humanoid police = (Humanoid) itr12.next();
            if (((MotionlessObject) self().motionlessPosition()).equals(police.
                    motionlessPosition()) ||
                ((MotionlessObject) self().motionlessPosition()).neighborhood().
                contains(police.motionlessPosition())) {
                return true;
            } else {
                Iterator itr3 = ((MotionlessObject) self().motionlessPosition()).
                                neighborhood().iterator();
                if (((MotionlessObject) itr3.next()).neighborhood().contains(
                        police.motionlessPosition()) ||
                    (itr3.hasNext() &&
                     ((MotionlessObject) itr3.next()).
                     neighborhood().contains(police.motionlessPosition())) ||
                    (itr3.hasNext() &&
                     ((MotionlessObject) itr3.next()).
                     neighborhood().contains(police.motionlessPosition())) ||
                    (itr3.hasNext() &&
                     ((MotionlessObject) itr3.next()).
                     neighborhood().contains(police.motionlessPosition()))) {
                    return true;
                }
            }
        }
        return false;
    }


    private void checkBloackade() throws ActionCommandException {

        if ((self().motionlessPosition() instanceof Road) &&
            ((Road) self().motionlessPosition()).passableLinesToHead() == 0 &&
            ((Road) self().motionlessPosition()).passableLinesToTail() == 0 &&
            isNear(world.nearPolice))

        {
            move(world.buildings);
        }
    }

    ArrayList deadArray = new ArrayList();


    protected void reportburriedness() {
        Humanoid deadHuman = null;
        int maxprio = 100;

        if (!world.nearHumanoid.isEmpty()) {
            for (Iterator it = world.nearHumanoid.iterator(); it.hasNext(); ) {
                Humanoid tempHumanoid;
                tempHumanoid = (Humanoid) it.next();
                if ((tempHumanoid instanceof Civilian) && tempHumanoid.hp() > 0) {
                    tell("c " + tempHumanoid.hp() + " " + tempHumanoid.damage() +
                         " " +
                         tempHumanoid.buriedness() + " " + tempHumanoid.id +
                         " " +
                         tempHumanoid.motionlessPosition().id);
                } else {
                    if (tempHumanoid != null &&
                        !deadArray.contains(tempHumanoid)) {
                        deadArray.add(tempHumanoid);
                        tell("die " + tempHumanoid.id);
                    }

                }
            }
        }

        if (!world.nearHumanoid.isEmpty()) {
            recordHuman bestHuman = new recordHuman();
            bestHuman.hp = ((Humanoid) world.nearHumanoid.get(0)).hp();
            bestHuman.damage = ((Humanoid) world.nearHumanoid.get(0)).damage();
            bestHuman.buridness = ((Humanoid) world.nearHumanoid.get(0)).
                                  buriedness();
            bestHuman.time = ((Humanoid) world.nearHumanoid.get(0)).time();
            bestHuman.self = (Humanoid) world.nearHumanoid.get(0);
            bestHuman.mlpos = ((Humanoid) world.nearHumanoid.get(0)).
                              motionlessPosition();
            if (bestHuman.mlpos instanceof Building &&
                !((Building) bestHuman.mlpos()).civilians.contains(bestHuman.
                    self)) {
                ((Building) bestHuman.mlpos()).setNewCivilian(bestHuman.self);
            }
            maxprio = bestHuman.priority(world.time(),
                                         world.ambulanceTeams.size());
            Iterator itr = world.nearHumanoid.iterator();
            itr.next();
            recordHuman bestHuman1 = new recordHuman();
            for (; itr.hasNext(); ) {
                Humanoid temp = (Humanoid) itr.next();
                if (temp.hp() == 0) {
                    deadHuman = temp;
                }
                int tempAmbulanceSize = 1;
                if (world.ambulanceTeams.size() > 1) {
                    tempAmbulanceSize = world.ambulanceTeams.size();
                }
                int futureHp = temp.hp() -
                               temp.buriedness() * temp.damage() /
                               (tempAmbulanceSize);
                if (temp.buriedness() > 0 && futureHp > 0 &&
                    !(brdReports.contains(temp))) {
                    bestHuman1.hp = temp.hp();
                    bestHuman1.damage = temp.damage();
                    bestHuman1.buridness = temp.buriedness();
                    bestHuman1.time = temp.time();
                    bestHuman1.mlpos = temp.motionlessPosition();
                    bestHuman1.self = temp;
                    if (bestHuman.mlpos instanceof Building &&
                        !((Building) bestHuman.mlpos()).civilians.contains(
                                bestHuman.self)) {
                        ((Building) bestHuman.mlpos()).setNewCivilian(bestHuman.
                                self);
                    }
                    if (bestHuman1.priority(world.time(),
                                            world.ambulanceTeams.size()) <
                        maxprio && bestHuman1.mlpos() != null) {
                        bestHuman = bestHuman1;
                        maxprio = bestHuman1.priority(world.time(),
                                world.ambulanceTeams.size());
                    }
                }
            }
            if (bestHuman.buridness > 0 && bestHuman != null && bestHuman.mlpos() != null &&
                bestHuman.hp > 0) {
                if (brdReports.contains(bestHuman.self)) {
                    brdReports.remove(bestHuman.self);
                }
                brdReports.add(bestHuman.self);

                tell("c " + bestHuman.hp + " " + bestHuman.damage + " " +
                     bestHuman.buridness + " " + bestHuman.self.id + " " +
                     bestHuman.mlpos().id);

            }
        }
    }

    protected void supplyWater() throws ActionCommandException {
        if (self().waterQuantity() == 0) {
            moveToRefuges();
        }
    }

    protected void extinguishOrMove(Collection fires) throws
            ActionCommandException {
        if (!fires.isEmpty()) {
            if (self().motionlessPosition() instanceof Building) {
                Building temp = (Building) self().motionlessPosition();
                if (temp.isBurning() || temp.isPutOut()) {
                    ArrayList arr = new ArrayList();
                    arr.add(temp);
                    extinguishNearFire(arr);
                } else {
                    extinguishNearFire(fires);
                }
            } else {
                extinguishNearFire(fires);
            }
            moveToEntrances(fires);
        }

    }

    protected void extinguishNearFire(Collection fires) throws
            ActionCommandException {
        Collection nearFires = isNearCnd.extract(fires);
        if (!nearFires.isEmpty()) {
            extinguish((Building) BURNING_TIME_PRP.min(nearFires));
        }

    }


    protected void moveToEntrances(Collection fires) throws
            ActionCommandException {
        ArrayList nearPointFire = new ArrayList();
        ArrayList nearFireBrigades = new ArrayList();
        nearFireBrigades.addAll(world.fireBrigades);
        nearFireBrigades.remove(world.get(self().id));
        if (!fires.isEmpty()) {

            for (Iterator it = fires.iterator(); it.hasNext(); ) {
                nearPointFire.addAll(((Building) it.next()).
                                     neighborBuildingwithoutE());
            }
            nearPointFire.removeAll(IS_BURNING_CND.extract(world.buildings));
            ArrayList temparr = new ArrayList();
            for (Iterator it1 = nearPointFire.iterator(); it1.hasNext(); ) {

                Building tempbuild = (Building) it1.next();
                for (Iterator it2 = nearFireBrigades.iterator(); it2.hasNext(); ) {
                    FireBrigade temp = (FireBrigade) it2.next();
                    if (temp.position() instanceof MotionlessObject) {
                        if (temp.motionlessPosition().equals(tempbuild.
                                motionlessPosition()) ||
                            temp.motionlessPosition().equals(tempbuild.
                                entrance())) {
                            temparr.add(tempbuild);
                        }
                    }
                }
            }
            nearPointFire.removeAll(temparr);

            if (!nearPointFire.isEmpty()) {
                move(nearPointFire);
            }

        }
    }

    protected void moveToRefuges() throws ActionCommandException {
        if (self().motionlessPosition() instanceof Refuge) {
            rest();
        }
        move(world.refuges);
    }

    protected void moveToAnother() throws Agent.ActionCommandException {
        moveToRefuges();
    }

    protected void correctZone(int numberOfZoneIn, int zoneIn) {
        int numberOfColumn = numberOfZoneIn / 2 + numberOfZoneIn % 2;
        int column = (zoneIn <= numberOfColumn) ? zoneIn :
                     zoneIn - numberOfColumn;
        int row = (zoneIn > numberOfColumn) ? 1 : 0;
        if (row == 1 && numberOfZoneIn % 2 == 1) {
            numberOfColumn = numberOfColumn - 1;
        }
        long maxYTemp = (row == 0) ?
                        world.minY() + ((world.maxY() - world.minY()) / 2) +
                        800 : world.maxY();
        long maxXTemp = world.minX() +
                        (column * (world.maxX() - world.minX()) /
                         numberOfColumn) +
                        650;
        long minXTemp = world.minX() +
                        ((column - 1) * (world.maxX() - world.minX()) /
                         numberOfColumn) - 650;
        long minYTemp = (row == 0) ? world.minY() :
                        world.minY() + ((world.maxY() - world.minY()) / 2) -
                        800;
        Iterator itr1 = world.roads.iterator();
        Road road = (Road) world.roads.iterator().next();
        for (itr1 = world.roads.iterator(); itr1.hasNext(); ) {
            road = (Road) itr1.next();
            if (road.x() >= minXTemp && road.x() <= maxXTemp &&
                road.y() <= maxYTemp && road.y() >= minYTemp) {
                world.updateBlockByAnother(road);
            }
        }
    }

    protected void hear(RealObject sender, String message) {
        StringTokenizer stFirst = new StringTokenizer(message, "*");
        for (; stFirst.hasMoreTokens(); ) {
            String messageTemp = stFirst.nextToken();
            StringTokenizer str = new StringTokenizer(messageTemp);
            String command = str.nextToken();
            if (command.equals("c")) {
                if (str.hasMoreTokens()) {
                    str.nextToken();
                }
                if (str.hasMoreTokens()) {
                    str.nextToken();
                }
                if (str.hasMoreTokens()) {
                    str.nextToken();
                }
                if (str.hasMoreTokens()) {
                    Humanoid reportedOthersHuman = (Humanoid) world.get(Integer.
                            parseInt(str.nextToken()));
                    if (!brdReports.contains(reportedOthersHuman)) {
                        brdReports.add(reportedOthersHuman);
                    }
                    if (str.hasMoreTokens()) {
                        MotionlessObject position = (MotionlessObject) world.
                                get(Integer.parseInt(str.nextToken()));
                        if (position instanceof Building &&
                            !((Building) position).civilians.contains(
                                    reportedOthersHuman)) {
                            ((Building) position).setNewCivilian(
                                    reportedOthersHuman);
                        }
                    }
                }
            }

            if (command.equals("clearZone:") && str.hasMoreTokens()) {
                int numberOfZoneIn = Integer.parseInt(str.nextToken());
                int zoneIn = Integer.parseInt(str.nextToken());
                correctZone(numberOfZoneIn, zoneIn);
              }
            if ((command.equals("clr") || command.equals("nmv")) &&
                str.hasMoreTokens()) {
                reportBlockadeOthers.add((MotionlessObject) world.get(Integer.
                        parseInt(str.nextToken())));
            }

        }
    }
}
